/* vi:set ts=8 sts=0 sw=8:
 * $Id: win.h,v 1.13 2000/05/20 17:56:11 kahn Exp kahn $
 *
 * Copyright (C) 1998 Andy C. Kahn
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __WIN_H__
#define __WIN_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <config.h>
#include "pred.h"

enum ef_wgt {
	Ef_wgt_toplev,
	Ef_wgt_dialog,
	Ef_wgt_button,
	Ef_wgt_checkbutton,
	Ef_wgt_radiobutton,
	Ef_wgt_entry,
	Ef_wgt_combo,
	Ef_wgt_clist,
	Ef_wgt_list,
	Ef_wgt_text,
	Ef_wgt_vbox,
	Ef_wgt_table,
	Ef_wgt_frame,
	Ef_wgt_notebook,
	Ef_wgt_spinbutton,
	Ef_wgt_optmenu,
	Ef_wgt_toolbar,
	Ef_wgt_label,
	Ef_wgt_menuitem,
	Ef_wgt_dont_care,
};
typedef enum ef_wgt ef_wgt_t;

enum findopt_e {
	FO_NONE			= 0,
	FO_PARTIAL_FNAME	= (1 << 0),
	FO_CASE_SENS		= (1 << 1),
	FO_SEARCH_SUBDIR	= (1 << 2),
	FO_LAST			= (1 << 31)
};
typedef enum findopt_e findopt_t;

struct ef_s {
	char *		glname;
	int		wgt_id;
	ef_wgt_t	wgt_type;
	GtkSignalFunc	cb;
	gpointer	data;
};
typedef struct ef_s ef_t;

enum ef_advops_id_e {
	Ef_advopts,		/* toplevel */
	Ef_exact_match,		/* checkbutton */
};
typedef enum ef_advops_id_e ef_advops_id_t;

/* Info on each directory in the current tree branch, to avoid
   getting stuck in symbolic link loops.  */
struct dir_id
{
  ino_t ino;
  dev_t dev;
};

/* originally from lib/idcache.c */
struct userid
{
  union
    {
      uid_t u;
      gid_t g;
    } id;
  char *name;
  struct userid *next;
};

struct win_s {
	GtkWidget **	mainwgts;	/* main window widgets */
	GtkWidget **	reswgts;	/* results/output window widgets */
	GtkWidget **	advwgts;	/* advanced option window widgets */
	GtkWidget **	filewgts;	/* multiple file widgets */
	GtkWidget **	dirwgts;	/* multiple directory widgets */
	int		findopts;	/* see findopt_t */
	int		advopts;	/* see advopt_t */
	gboolean	stop_search;	/* stop current search? */
	gboolean	is_searching;	/* search in progress? */
	gboolean	close_window;	/* win close requested */
	guint		num_found;	/* num files found */
	guint		num_errs;	/* num errors during search */

	GtkWidget *	msgbar;		/* convenience pointer */
	char *		lastmsg;	/* last message in message bar */
	guint		timeout_id;	/* msgbar timeout id */
	GtkProgress *	progbar;	/* convenience pointer */


	/*** various statics/globals used in the original find code ***/

	/* originally in lib/idcache.c */
	struct userid *user_alist;

	/* The members of this list have names not in the local passwd file. */
	struct userid *nouser_alist;

	/* Use the same struct as for userids.  */
	struct userid *group_alist;
	struct userid *nogroup_alist;


	/* originally in lib/listfile.c */
	int kilobytes;	/* -1 = uninitialized, 0 = 512, 1 = 1024. */

	
	/* originally in pred.c */
	gboolean	launch_first_time;	/* used in launch() */

	/* originally in fstype.c */
	gboolean	fstype_known;		/* filesystem_type() */
	char *		cur_fstype;		/* filesystem_type() */
	dev_t		cur_dev;		/* filesystem_type() */

	/* originally in find.c */
	/* Name this program was run with. */
	char *program_name;

	/* All predicates for each path to process. */
	struct predicate *predicates;

	/* The last predicate allocated. */
	struct predicate *last_pred;

	/* The root of the evaluation tree. */
	struct predicate *eval_tree;

	/* If true, process directory b4 contents.  True unless -depth given. */
	gboolean do_dir_first;

	/* If >=0, don't descend more than this many levels of subdirs. */
	int maxdepth;

	/* If >=0, don't process files above this level. */
	int mindepth;

	/* Current depth; 0 means current path is a command line arg. */
	int curdepth;

	/* Seconds between 00:00 1/1/70 and either one day before now
	   (the default), or the start of today (if -daystart is given). */
	time_t cur_day_start;

	/* If true, cur_day_start has been adjusted to the start of the day. */
	gboolean full_days;

	/* If true, do not assume that files in directories with nlink == 2
	   are non-directories. */
	gboolean no_leaf_check;

	/* If true, don't cross filesystem boundaries. */
	gboolean stay_on_filesystem;

	/* If true, don't descend past current directory.
	   Can be set by -prune, -maxdepth, and -xdev/-mount. */
	gboolean stop_at_current_level;

#ifndef HAVE_FCHDIR
	/* The full path of the initial working directory.  */
	char *starting_dir;
#else
	/* A file descriptor open to the initial working directory.
	   Doing it this way allows us to work when the i.w.d. has
	   unreadable parents.  */
	int starting_desc;
#endif

	/* If true, we have called stat on the current path. */
	gboolean have_stat;

	/* The file being operated on, relative to the current directory.
	   Used for stat, readlink, and opendir.  */
	char *rel_pathname;

	/* Length of current path. */
	int path_length;

	/* true if following symlinks.  Should be consistent with xstat.  */
	gboolean dereference;

	/* Pointer to the function used to stat files. */
	int (*xstat) ();

	/* Status value to return to system. */
	int exit_status;

	struct dir_id *dir_ids;
	/* Entries allocated in `dir_ids'.  */
	int dir_alloc;
	/* Index in `dir_ids' of directory currently being searched.
	   This is always the last valid entry.  */
	int dir_curr;

	dev_t root_dev;	/* Device ID of current argument pathname. */

	/* originally in parser.c */
#ifdef CACHE_IDS
	/* Boolean: if uid_unused[n] is nonzero, then UID n has no passwd
	 * entry. */
	char *uid_unused;

	/* Number of elements in `uid_unused'. */
	unsigned uid_allocated;

	/* Similar for GIDs and group entries. */
	char *gid_unused;
	unsigned gid_allocated;
#endif
}; /* win_s */
typedef struct win_s win_t;


extern void	win_fd_entry_sensitive(win_t *, gboolean, gboolean);
extern void	win_fd_entry_setclear(win_t *, gboolean, gboolean);
extern GtkWidget *win_main_toplev(win_t *w);
extern void	win_progbar_update(win_t *w);
extern void	win_new_cb(GtkWidget *wgt, gpointer cbdata);
extern void	win_close_cb(GtkWidget *wgt, gpointer cbdata);
extern win_t *	win_new(void);
extern void	win_wprint(win_t *w, char *path, struct stat *sb,
			   struct predicate *pred);
extern void	win_widgets_setup(GladeXML *xml, win_t *w,
				  GtkWidget **wgts, ef_t *efp);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif	/* __WIN_H__ */
